pub mod db;
use actix_web::{get, post, web, Error, HttpResponse, Responder, Result};
use db::Db;
use serde::{Deserialize, Serialize};

#[derive(Serialize)]
struct ArrayResponse<T> {
    result: Vec<T>,
}

#[derive(Debug, Serialize, Deserialize)]
pub struct AddColorRequest {
    pub name: String,
}

#[get("/api/cats")]
pub async fn get_cats(db: web::Data<Db>) -> Result<impl Responder> {
    let cats = db.get_cats().await;
    return Ok(HttpResponse::Ok().json(cats.map_err(Error::from)?));
}

#[get("/api/cat/random")]
pub async fn get_random_cat(db: web::Data<Db>) -> Result<impl Responder> {
    let cat = db.get_random_cat().await;
    return Ok(HttpResponse::Ok().json(cat.map_err(Error::from)?));
}

#[get("/api/colors")]
pub async fn get_colors(db: web::Data<Db>) -> Result<impl Responder> {
    let colors = db.get_colors().await;
    return Ok(HttpResponse::Ok().json(colors.map_err(Error::from)?));
}

#[derive(Debug, Serialize, Deserialize)]
pub struct AddCatRequest {
    pub name: String,
    pub color: String,
}

#[post("/api/add/cat")]
pub async fn add_cat(db: web::Data<Db>, cat: web::Json<AddCatRequest>) -> Result<impl Responder> {
    let _cat = cat.into_inner();
    let cat = db.add_cat(_cat.name, _cat.color).await;
    return Ok(HttpResponse::Ok().json(cat.map_err(Error::from)?));
}

#[post("/api/add/color")]
pub async fn add_color(
    db: web::Data<Db>,
    cat: web::Json<AddColorRequest>,
) -> Result<impl Responder> {
    let _color = cat.into_inner();
    let cat = db.add_color(_color.name).await;
    return Ok(HttpResponse::Ok().json(cat.map_err(Error::from)?));
}
